# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals
)

import os
from subprocess import PIPE, Popen

from boltons.cacheutils import cachedproperty
from packaging.specifiers import SpecifierSet

from . import logger


class PackageManager(object):
    """ Package manager definition. """

    # Fully qualified path to the package manager CLI.
    cli_path = None

    # Systematic options passed to package manager CLI. Might be of use to
    # force silencing or high verbosity for instance.
    cli_args = []

    # Version requirement specifier.
    requirement = None

    def __init__(self):
        # List all available updates and their versions.
        self.updates = []
        self.error = None

    @cachedproperty
    def version(self):
        """ Parsed and normalized package manager's own version.

        Returns an instance of packaging.Version or None if version doesn't
        matter.
        """
        return None

    @cachedproperty
    def version_string(self):
        """ String representation of the canonical version. """
        return "{}".format(self.version) if self.version else None

    @cachedproperty
    def id(self):
        """ Return package manager's ID. Defaults based on class name.

        This ID must be unique among all package manager definitions and
        lower-case as they're used as feature flags for the mpm CLI.
        """
        return self.__class__.__name__.lower()

    @cachedproperty
    def name(self):
        """ Return package manager's common name. Defaults based on class name.
        """
        return self.__class__.__name__

    @cachedproperty
    def exists(self):
        """ Is the package manager CLI exist on the system? """
        if not os.path.isfile(self.cli_path):
            logger.debug("{} not found.".format(self.cli_path))
            return False
        return True

    @cachedproperty
    def executable(self):
        """ Is the package manager CLI can be executed by the current user? """
        if not os.access(self.cli_path, os.X_OK):
            logger.debug("{} not executable.".format(self.cli_path))
            return False
        return True

    @cachedproperty
    def supported(self):
        """ Is the package manager match the version requirement? """
        if self.version and self.requirement:
            if self.version not in SpecifierSet(self.requirement):
                logger.debug(
                    "{} {} doesn't fit the '{}' version requirement.".format(
                        self.id, self.version, self.requirement))
                return False
        return True

    @cachedproperty
    def available(self):
        """ Is the package manager available and ready-to-use on the system?

        Returns True only if the main CLI:
            1 - exists,
            2 - is executable, and
            3 - match the version requirement.
        """
        return self.exists and self.executable and self.supported

    def run(self, cmd):
        """ Run a shell command, return the output and keep error message. """
        self.error = None
        assert isinstance(cmd, list)
        logger.debug("Running `{}`...".format(' '.join(cmd)))
        process = Popen(cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True)
        output, error = process.communicate()
        if process.returncode != 0 and error:
            self.error = error.decode('utf-8')
            logger.warning(self.error)
        return output.decode('utf-8')

    def sync(self):
        """ Fetch latest versions of installed packages.

        Returns a list of dict with package name, current installed version and
        latest upgradeable version.
        """
        logger.info('Sync {} package info...'.format(self.id))

    def update_cli(self, package_name=None):
        """ Return a bash-compatible full-CLI to update a package. """
        raise NotImplementedError

    def update(self, package_name=None):
        """ Effectively perform an update of provided package. """
        return self.run(self.update_cli(package_name))

    def update_all_cli(self):
        """ Return a bash-compatible full-CLI to update all packages. """
        raise NotImplementedError

    def update_all(self):
        """ Effectively perform a full update of all outdated packages.

        If the manager doesn't implements a full update one-liner, then
        fall-back to calling single-package update one by one.
        """
        try:
            return self.run(self.update_all_cli())
        except NotImplementedError:
            logger.warning(
                "{} doesn't seems to implement a full update subcommand. Call "
                "single-package update CLI one by one.".format(self.id))
            output = []
            for package in self.updates:
                output.append(self.update(package['name']))
            return '\n'.join(output)

    @staticmethod
    def bitbar_cli_format(full_cli):
        """ Format a bash-runnable full-CLI with parameters into bitbar schema.
        """
        cmd, params = full_cli.strip().split(' ', 1)
        bitbar_cli = "bash={}".format(cmd)
        for index, param in enumerate(params.split(' ')):
            bitbar_cli += " param{}={}".format(index + 1, param)
        return bitbar_cli



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals
)

import json

from boltons.cacheutils import cachedproperty
from packaging.version import parse as parse_version

from ..base import PackageManager


class Homebrew(PackageManager):

    cli_path = '/usr/local/bin/brew'

    requirement = '>= 1.0.0'

    @cachedproperty
    def id(self):
        return "brew"

    @cachedproperty
    def version(self):
        metadata = self.run([self.cli_path] + self.cli_args + ['--version'])
        return parse_version(metadata.split()[1])

    def sync(self):
        """ Fetch latest Homebrew formulas.

        Sample of brew output:

            $ brew outdated --json=v1
            [
              {
                "name": "cassandra",
                "installed_versions": [
                  "3.5"
                ],
                "current_version": "3.7"
              },
              {
                "name": "vim",
                "installed_versions": [
                  "7.4.1967"
                ],
                "current_version": "7.4.1993"
              },
              {
                "name": "youtube-dl",
                "installed_versions": [
                  "2016.07.06"
                ],
                "current_version": "2016.07.09.1"
              }
            ]
        """
        super(Homebrew, self).sync()

        self.run([self.cli_path] + self.cli_args + ['update'])

        # List available updates.
        output = self.run(
            [self.cli_path] + self.cli_args + ['outdated', '--json=v1'])
        if not output:
            return

        for pkg_info in json.loads(output):
            self.updates.append({
                'name': pkg_info['name'],
                # Only keeps the highest installed version.
                'installed_version': max(pkg_info['installed_versions']),
                'latest_version': pkg_info['current_version']})

    def update_cli(self, package_name=None):
        cmd = [self.cli_path] + self.cli_args + ['upgrade', '--cleanup']
        if package_name:
            cmd.append(package_name)
        return cmd

    def update_all_cli(self):
        return self.update_cli()


class HomebrewCask(Homebrew):

    """ Cask is now part of Homebrew's core and extend it. """

    cli_args = ['cask']

    # 'cask install' doesn't update to the latest package version, we need to
    # call 'cask reinstall' instead since 1.1.0.
    requirement = '>= 1.1.0'

    @cachedproperty
    def id(self):
        return "cask"

    @cachedproperty
    def name(self):
        return "Homebrew Cask"

    def sync(self):
        """ Fetch latest formulas and their metadata.

        Sample of brew cask output:

            $ brew cask list --versions
            aerial 1.2beta5
            android-file-transfer latest
            audacity 2.1.2
            bitbar 1.9.2
            firefox 49.0.1
            flux 37.7
            gimp 2.8.18-x86_64
            java 1.8.0_112-b16

            $ brew cask info aerial
            aerial: 1.2beta5
            https://github.com/JohnCoates/Aerial
            /usr/local/Caskroom/aerial/1.2beta5 (18 files, 6.6M)
            From: https://github.com/caskroom/homebrew-cask/blob/master/Casks/aerial.rb
            ==> Name
            Aerial Screensaver
            ==> Artifacts
            Aerial.saver (screen_saver)

            $ brew cask info firefox
            firefox: 50.0.1
            https://www.mozilla.org/firefox/
            /usr/local/Caskroom/firefox/49.0.1 (107 files, 185.3M)
            From: https://github.com/caskroom/homebrew-cask/blob/master/Casks/firefox.rb
            ==> Name
            Mozilla Firefox
            ==> Artifacts
            Firefox.app (app)

            $ brew cask info prey
            prey: 1.6.3
            https://preyproject.com/
            Not installed
            From: https://github.com/caskroom/homebrew-cask/blob/master/Casks/prey.rb
            ==> Name
            Prey
            ==> Artifacts
            prey-mac-1.6.3-x86.pkg (pkg)
            ==> Caveats
            Prey requires your API key, found in the bottom-left corner of
            the Prey web account Settings page, to complete installation.
            The API key may be set as an environment variable as follows:

              API_KEY="abcdef123456" brew cask install prey

            $ brew cask info ubersicht
            ubersicht: 1.0.44
            http://tracesof.net/uebersicht/
            Not installed
            From: https://github.com/caskroom/homebrew-cask/blob/master/Casks/ubersicht.rb
            ==> Name
            Übersicht
            ==> Artifacts
            Übersicht.app (app)
        """
        PackageManager().sync()

        # `brew cask update` is just an alias to `brew update`.
        self.run([self.cli_path] + self.cli_args + ['update'])

        # List installed packages.
        output = self.run(
            [self.cli_path] + self.cli_args + ['list', '--versions'])

        # Inspect package one by one as `brew cask list` is not reliable. See:
        # https://github.com/caskroom/homebrew-cask/blob/master/doc
        # /reporting_bugs/brew_cask_list_shows_wrong_information.md
        for installed_pkg in output.strip().split('\n'):
            if not installed_pkg:
                continue
            infos = installed_pkg.split(' ', 1)
            name = infos[0]

            # Use heuristics to guess installed version.
            versions = infos[1] if len(infos) > 1 else ''
            versions = sorted([
                v.strip() for v in versions.split(',') if v.strip()])
            if len(versions) > 1 and 'latest' in versions:
                versions.remove('latest')
            version = versions[-1] if versions else '?'

            # TODO: Support packages removed from repository (reported with a
            # `(!)` flag). See: https://github.com/caskroom/homebrew-cask/blob
            # /master/doc/reporting_bugs
            # /uninstall_wrongly_reports_cask_as_not_installed.md

            # Inspect the package closer to evaluate its state.
            output = self.run([self.cli_path] + self.cli_args + ['info', name])

            latest_version = output.split('\n')[0].split(' ')[1]

            # Skip already installed packages.
            if version == latest_version:
                continue

            self.updates.append({
                'name': name,
                'installed_version': version,
                'latest_version': latest_version})

    def update_cli(self, package_name):
        """ Install a package.

        TODO: wait for https://github.com/caskroom/homebrew-cask/issues/22647
        so we can force a cleanup in one go, as we do above with vanilla
        Homebrew.
        """
        return [self.cli_path] + self.cli_args + ['reinstall', package_name]

    def update_all_cli(self):
        """ Cask has no way to update all outdated packages.

        See: https://github.com/caskroom/homebrew-cask/issues/4678
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

""" Registration, indexing and cache of package manager definitions. """

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals
)

import inspect
from glob import glob
from importlib import import_module
from os import path

from boltons.cacheutils import LRI, cached

from .. import logger
from ..base import PackageManager


# Use a cache to keep re-computing the global pool of registered manager
# definitions.
@cached(LRI(max_size=1))
def pool():
    """ Search for package manager definitions locally and return a dict.

    Is considered valid package manager definitions classes which:
        1 - are sub-classes of PackageManager, and
        2 - are located in files at the same level or below this one, and
        3 - have a defined cli_path property.
    """
    register = {}

    here = path.dirname(path.abspath(__file__))

    for py_file in glob(path.join(here, '*.py')):
        module_name = path.splitext(path.basename(py_file))[0]
        logger.debug("Search manager definitions in {}".format(py_file))
        module = import_module('.{}'.format(module_name), package=__package__)

        for _, klass in inspect.getmembers(module, inspect.isclass):
            if issubclass(klass, PackageManager) and (
                    klass.cli_path is not None):
                logger.debug("Found {!r}".format(klass))
                manager = klass()
                register[manager.id] = manager
            else:
                logger.debug(
                    "{!r} is not a valid manager definition".format(klass))

    return register



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals
)

import unittest

from meta_package_manager import PY3
from meta_package_manager.managers import pool

if PY3:
    basestring = (str, bytes)


class TestDefinition(unittest.TestCase):

    """ Test the definition of all package managers. """

    def test_ascii_id(self):
        """ All package manager IDs should be short ASCII strings. """
        for manager_id in pool():
            self.assertRegexpMatches(manager_id, r'[a-z0-9]+')

    def test_number(self):
        """ Check all supported package managers are accounted for. """
        self.assertEqual(len(pool()), 8)

    def test_cli_path_type(self):
        """ Check that definitions returns the CLI path as a string. """
        for manager in pool().values():
            self.assertIsInstance(manager.cli_path, basestring)

    def test_cli_args_type(self):
        """ Check that definitions returns CLI args as a list. """
        for manager in pool().values():
            self.assertIsInstance(manager.cli_args, list)

    def test_cli_type(self):
        """ Check that all methods returning a CLI is a list. """
        for manager in pool().values():
            self.assertIsInstance(manager.update_cli('dummy_package_id'), list)

            # Update-all CLI are allowed to raise a particular error.
            try:
                result = manager.update_all_cli()
            except Exception as excpt:
                self.assertIsInstance(excpt, NotImplementedError)
            else:
                self.assertIsInstance(result, list)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import absolute_import, division, print_function

import logging
from json import dumps as json_dumps
from operator import itemgetter

import click
import click_log
from tabulate import tabulate

from . import __version__, logger
from .managers import pool

# Output rendering modes. From machine to human-readable.
RENDERING_MODES = {
    'json': 'json',
    # Mapping of table formating options to tabulate's parameters.
    'plain': 'plain',
    'simple': 'simple',
    'fancy': 'fancy_grid'}


def json(data):
    """ Utility function to render data structure into pretty printed JSON. """
    return json_dumps(data, sort_keys=True, indent=4, separators=(',', ': '))


@click.group(invoke_without_command=True)
@click_log.init(logger)
@click_log.simple_verbosity_option(
    default='INFO', metavar='LEVEL',
    help='Either CRITICAL, ERROR, WARNING, INFO or DEBUG. Defaults to INFO.')
@click.option(
    '-o', '--output-format', type=click.Choice(RENDERING_MODES),
    default='fancy', help="Rendering mode of the output. Defaults to fancy.")
@click.version_option(__version__)
@click.pass_context
def cli(ctx, output_format):
    """ CLI for multi-package manager updates and upgrades. """
    level = click_log.get_level()
    level_name = logging._levelNames.get(level, level)
    logger.debug('Verbosity set to {}.'.format(level_name))

    # Print help screen and exit if no sub-commands provided.
    if ctx.invoked_subcommand is None:
        click.echo(ctx.get_help())
        ctx.exit()

    # Load up global options to the context.
    ctx.obj = {
        'rendering': RENDERING_MODES[output_format]}

    # Silent all log message in JSON rendering mode unless it's at debug level.
    if ctx.obj['rendering'] == 'json' and level_name != 'DEBUG':
        click_log.set_level(logging.CRITICAL * 2)


@cli.command(short_help='List supported package managers and their location.')
@click.pass_context
def managers(ctx):
    """ List all supported package managers and their presence on the system.
    """
    rendering = ctx.obj['rendering']

    # Machine-friendly data rendering.
    if rendering == 'json':
        fields = [
            'name', 'id', 'cli_path', 'exists', 'executable', 'version_string',
            'supported', 'available']
        # JSON mode use print to output data because the logger is disabled.
        print(json({
            manager_id: {fid: getattr(manager, fid) for fid in fields}
            for manager_id, manager in pool().items()}))
        return

    # Human-friendly content rendering.
    table = []
    for manager_id, manager in pool().items():
        table.append([
            manager.name,
            manager_id,
            manager.cli_path,
            u'✅' if manager.exists else '',
            u'✅' if manager.executable else '',
            u"{}  {}".format(
                u'✅' if manager.supported else u'❌',
                manager.version if manager.version else '')
            if manager.exists else ''])
    table = [[
        'Package manager', 'ID', 'CLI path', 'Found', 'Executable',
        'Version']] + sorted(table, key=itemgetter(1))
    logger.info(tabulate(table, tablefmt=rendering, headers='firstrow'))


@cli.command(short_help='Sync local package info.')
@click.pass_context
def sync(ctx):
    """ Sync local package metadata and info from external sources. """
    for manager_id, manager in pool().items():

        # Filters-out inactive managers.
        if not manager.available:
            logger.warning('Skip unavailable {} manager.'.format(manager_id))
            continue

        manager.sync()


@cli.command(short_help='List available updates.')
@click.pass_context
def outdated(ctx):
    """ List available package updates and their versions for each manager. """
    # Build-up a global list of outdated packages per manager.
    outdated = {}

    for manager_id, manager in pool().items():

        # Filters-out inactive managers.
        if not manager.available:
            logger.warning('Skip unavailable {} manager.'.format(manager_id))
            continue

        # Force a sync to get the freshest updates.
        manager.sync()

        if manager.error:
            logger.error(manager.error)

        outdated[manager_id] = {
            'id': manager_id,
            'name': manager.name,
            'packages': [{
                'name': pkg_info['name'],
                'id': pkg_info['name'],
                'installed_version': pkg_info['installed_version'],
                'latest_version': pkg_info['latest_version']}
                for pkg_info in manager.updates]}

    rendering = ctx.obj['rendering']

    # Machine-friendly data rendering.
    if rendering == 'json':
        # JSON mode use print to output data because the logger is disabled.
        print(json(outdated))
        return

    # Human-friendly content rendering.
    table = []
    for manager_id, outdated_pkg in outdated.items():
        table += [[
            pkg_info['name'],
            pkg_info['id'],
            manager_id,
            pkg_info['installed_version'],
            pkg_info['latest_version']]
            for pkg_info in outdated_pkg['packages']]

    def sort_method(line):
        """ Force sorting by lower-cased package ID first, then manager ID. """
        return line[1].lower(), line[2]

    # Sort and print table.
    table = [[
        'Package name', 'ID', 'Manager', 'Installed version',
        'Latest version']] + sorted(table, key=sort_method)
    logger.info(tabulate(table, tablefmt=rendering, headers='firstrow'))
    # Print statistics.
    manager_stats = {
        manager_id: len(packages) for manager_id, packages in outdated.items()}
    total_outdated = sum(manager_stats.values())
    per_manager_totals = ', '.join([
        '{} from {}'.format(v, k) for k, v in sorted(manager_stats.items())])
    if per_manager_totals:
        per_manager_totals = ' ({})'.format(per_manager_totals)
    logger.info('{} outdated package{} found{}.'.format(
        total_outdated,
        's' if total_outdated > 1 else '',
        per_manager_totals))


@cli.command(short_help='Update all packages.')
@click.pass_context
def update(ctx):
    """ Perform a full package update on all available managers. """
    for manager_id, manager in pool().items():

        # Filters-out inactive managers.
        if not manager.available:
            logger.warning('Skip unavailable {} manager.'.format(manager_id))
            continue

        logger.info(
            'Updating all outdated packages from {}...'.format(manager_id))
        output = manager.update_all()
        logger.info(output)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals
)

import re

from boltons.cacheutils import cachedproperty
from packaging.version import parse as parse_version

from ..base import PackageManager


class MAS(PackageManager):

    cli_path = '/usr/local/bin/mas'

    # 'mas outdated' output has been changed in 1.3.1: https://github.com
    # /mas-cli/mas/commit/ca72ee42b1c5f482513b1d2fbf780b0bf3d9618b
    requirement = '>= 1.3.1'

    def __init__(self):
        super(MAS, self).__init__()
        self.map = {}

    @cachedproperty
    def name(self):
        return "Mac AppStore"

    @cachedproperty
    def version(self):
        return parse_version(self.run([self.cli_path, 'version']))

    def sync(self):
        super(MAS, self).sync()

        output = self.run([self.cli_path] + self.cli_args + ['outdated'])

        if not output:
            return

        regexp = re.compile(r'(\d+) (.*) \((\S+) -> (\S+)\)$')
        for application in output.split('\n'):
            if not application:
                continue
            _id, name, installed_version, latest_version = regexp.match(
                application).groups()
            self.map[name] = _id
            self.updates.append({
                'name': name,
                'latest_version': latest_version,
                # Normalize unknown version. See: https://github.com/mas-cli
                # /mas/commit/1859eaedf49f6a1ebefe8c8d71ec653732674341
                'installed_version': (
                    installed_version if installed_version != 'unknown'
                    else '')})

    def update_cli(self, package_name):
        if package_name not in self.map:
            return []
        return [self.cli_path] + self.cli_args + [
            'install', self.map[package_name]]

    def update_all_cli(self):
        return [self.cli_path] + self.cli_args + ['upgrade']



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals
)

import unittest

from click.testing import CliRunner

from meta_package_manager.cli import cli


class CLITestCase(unittest.TestCase):

    def setUp(self):
        self.runner = CliRunner()


class TestCLI(CLITestCase):

    def test_main_help(self):
        result = self.runner.invoke(cli, ['--help'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("--help", result.output)

    def test_verbosity(self):
        result = self.runner.invoke(cli, ['--verbosity', 'DEBUG'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("debug:", result.output)

        result = self.runner.invoke(cli, ['--verbosity', 'INFO'])
        self.assertEqual(result.exit_code, 0)
        self.assertNotIn("debug:", result.output)


class TestCLIManagers(CLITestCase):

    def test_main_help(self):
        result = self.runner.invoke(cli, ['managers', '--help'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("--help", result.output)

    def test_verbosity(self):
        result = self.runner.invoke(cli, ['--verbosity', 'DEBUG', 'managers'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("debug:", result.output)

        result = self.runner.invoke(cli, ['--verbosity', 'INFO', 'managers'])
        self.assertEqual(result.exit_code, 0)
        self.assertNotIn("debug:", result.output)

    def test_simple_call(self):
        result = self.runner.invoke(cli, ['managers'])
        self.assertEqual(result.exit_code, 0)


class TestCLISync(CLITestCase):

    def test_main_help(self):
        result = self.runner.invoke(cli, ['sync', '--help'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("--help", result.output)

    def test_verbosity(self):
        result = self.runner.invoke(cli, ['--verbosity', 'DEBUG', 'sync'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("debug:", result.output)

        result = self.runner.invoke(cli, ['--verbosity', 'INFO', 'sync'])
        self.assertEqual(result.exit_code, 0)
        self.assertNotIn("debug:", result.output)

    def test_simple_call(self):
        result = self.runner.invoke(cli, ['sync'])
        self.assertEqual(result.exit_code, 0)


class TestCLIOutdated(CLITestCase):

    def test_main_help(self):
        result = self.runner.invoke(cli, ['outdated', '--help'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("--help", result.output)

    def test_verbosity(self):
        result = self.runner.invoke(cli, ['--verbosity', 'DEBUG', 'outdated'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("debug:", result.output)

        result = self.runner.invoke(cli, ['--verbosity', 'INFO', 'outdated'])
        self.assertEqual(result.exit_code, 0)
        self.assertNotIn("debug:", result.output)

    def test_simple_call(self):
        result = self.runner.invoke(cli, ['outdated'])
        self.assertEqual(result.exit_code, 0)


class TestCLIUpdate(CLITestCase):

    def test_main_help(self):
        result = self.runner.invoke(cli, ['update', '--help'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("--help", result.output)

    def test_verbosity(self):
        result = self.runner.invoke(cli, ['--verbosity', 'DEBUG', 'update'])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("debug:", result.output)

        result = self.runner.invoke(cli, ['--verbosity', 'INFO', 'update'])
        self.assertEqual(result.exit_code, 0)
        self.assertNotIn("debug:", result.output)

    def test_simple_call(self):
        result = self.runner.invoke(cli, ['update'])
        self.assertEqual(result.exit_code, 0)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 Kevin Deldycke <kevin@deldycke.com>
#                    and contributors.
# All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

""" Expose package-wide elements. """

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals
)

import logging
import os
import sys


__version__ = '1.11.0'


PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3


# We only support macOS for now.
if sys.platform != 'darwin':
    raise RuntimeError(
        "{} platform is not supported. Only macOS is.".format(sys.platform))


logger = logging.getLogger(__name__)


# OS X does not put /usr/local/bin or /opt/local/bin in the PATH for GUI apps.
# For some package managers this is a problem. Additioanlly Homebrew and
# Macports are using different pathes.  So, to make sure we can always get to
# the necessary binaries, we overload the path.  Current preference order would
# equate to Homebrew, Macports, then System.
os.environ['PATH'] = ':'.join(['/usr/local/bin',
                               '/usr/local/sbin',
                               '/opt/local/bin',
                               '/opt/local/sbin',
                               os.environ.get('PATH', '')])
